@@ -59,6 +59,8 @@ class OdtWriter(writer.Writer):
         section.
     :param bool make_toc: If ``True`` (default) then generate Table of
         Contents.
+    :param bool events_without_dates: If ``True`` (default) then show events
+        that have no associated dates.
     :param Size page_width: Page width of the produced document.
     :param Size page_height: Page height of the produced document.
     :param Size margin_left: Left page margin of the produced document.
@@ -75,6 +77,7 @@ class OdtWriter(writer.Writer):
                  encoding_errors="strict",
                  sort_order=model.ORDER_SURNAME_GIVEN, name_fmt=0,
                  make_images=True, make_stat=True, make_toc=True,
+                 events_without_dates=True,
                  page_width="6in", page_height="9in",
                  margin_left="0.5in", margin_right="0.5in",
                  margin_top="0.5in", margin_bottom="0.25in",
@@ -85,7 +88,8 @@ class OdtWriter(writer.Writer):
                                encoding_errors=encoding_errors,
                                sort_order=sort_order, name_fmt=name_fmt,
                                make_images=make_images, make_stat=make_stat,
-                               make_toc=make_toc)
+                               make_toc=make_toc,
+                               events_without_dates=events_without_dates)
 
         self._output = output
         self._image_width = Size(image_width)
@@ -434,20 +438,19 @@ class OdtWriter(writer.Writer):
             self.doc.save(self._output)
 
     def _getImageFragment(self, image_data):
-        '''Returns <img> HTML fragment for given image data (byte array).
+        '''Adds Image to the document as person's picture.
         '''
 
-        imgfile = io.BytesIO(image_data)
-        img = Image.open(imgfile)
+        img = Image.open(io.BytesIO(image_data))
         filename = u"Pictures/" + \
-            hashlib.sha1(image_data).hexdigest() + '.' + img.format
+            hashlib.sha1(image_data).hexdigest() + '.' + img.format.lower()
 
         # calculate size of the frame
         maxsize = (self._image_width.inches,
                    self._image_height.inches)
         w, h = utils.resize(img.size, maxsize)
         frame = draw.Frame(width="%.3fin" % w, height="%.3fin" % h)
-        imgref = self.doc.addPicture(filename, "image/" + img.format,
+        imgref = self.doc.addPicture(filename, utils.img_mime_type(img),
                                      image_data)
         frame.addElement(draw.Image(href=imgref))
         return frame
