@@ -5,6 +5,7 @@ from __future__ import absolute_import, division, print_function
 
 __all__ = ["Writer"]
 
+import locale
 import logging
 
 from .events import indi_attributes, indi_events, family_events
@@ -61,11 +62,14 @@ class Writer(object):
         section.
     :param bool make_toc: If ``True`` (default) then generate Table of
         Contents.
+    :param bool events_without_dates: If ``True`` (default) then show events
+        that have no associated dates.
     """
 
     def __init__(self, flocator, tr, encoding=None, encoding_errors="strict",
                  sort_order=model.ORDER_SURNAME_GIVEN, name_fmt=0,
-                 make_images=True, make_stat=True, make_toc=True):
+                 make_images=True, make_stat=True, make_toc=True,
+                 events_without_dates=True):
 
         self._floc = flocator
         self._encoding = encoding
@@ -75,6 +79,7 @@ class Writer(object):
         self._make_images = make_images
         self._make_stat = make_stat
         self._make_toc = make_toc
+        self._events_without_dates = events_without_dates
         self._tr = tr
 
     def save(self):
@@ -110,7 +115,7 @@ class Writer(object):
             indis.append(indi)
 
         # loop over all individuals
-        indis.sort(key=lambda x: x.name.order(self._sort_order))
+        indis.sort(key=self._indi_sort_key)
         for person in indis:
 
             name = name_fmt(person.name, self._name_fmt)
@@ -238,6 +243,17 @@ class Writer(object):
         # finish
         self._finalize()
 
+    def _indi_sort_key(self, indi):
+        """Return name ordering key for individual
+        """
+        # make key from name, this is a tuple of unicode strings
+        key = indi.name.order(self._sort_order)
+
+        # we want locale-aware ordering
+        key = tuple(locale.strxfrm(x) for x in key)
+
+        return key
+
     def _events(self, person):
         """Returns a list of events for a given person.
 
@@ -290,15 +306,24 @@ class Writer(object):
                              evt.note]
                     events += [(evt.date, facts)]
 
-        # only use events with dates
-        events = [evt for evt in events if evt[0]]
+        def _date_key(event):
+            "Return event date, used for comparison"
+            date = event[0]
+            if date is None:
+                # use date in the future
+                date = model.DateValue()
+            return date
 
-        # order events
+        # order events (only those with dates)
         sevents = []
-        for date, facts in sorted(events):
+        for date, facts in sorted(events, key=_date_key):
             facts = [fact for fact in facts if fact]
             facts = u"; ".join(facts)
-            sevents += [(self._tr.tr_date(date), facts)]
+            if date is None:
+                if self._events_without_dates:
+                    sevents += [(self._tr.tr(TR("Event Date Unknown")), facts)]
+            else:
+                sevents += [(self._tr.tr_date(date), facts)]
 
         return sevents
 
